#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import Drawing, _DrawingEditorMixin, String, Rect
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.pdfbase.pdfmetrics import registerFont
from reportlab.lib.colors import red

class TestArialUni(_DrawingEditorMixin,Drawing):
	def __init__(self,width=400,height=800,*args,**kw):
		DataAwareDrawing.__init__(self,width,height,*args,**kw)
		registerFont(TTFont('arialuni','arialuni.ttf'))
		self._add(self,String(10,10,'text \xd0\x96 \xe7\xa4\xba \xe6\xbc\xb9',fontName='arialuni',fontSize=36),name='s0',validate=None,desc=None)
		bb = self.s0.getBounds()
		self._add(self,Rect(bb[0],bb[1],bb[2]-bb[0],bb[3]-bb[1],fillColor=None,strokeWidth=1,strokeColor=red),name=None,validate=None,desc=None)
		self.__dict__['preview'] = 0
		self._add(self,String(10,20,"""u0020=  u0021=! u0022=" u0023=# u0024=$ u0025=% u0026=&""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,32,"""u0027=' u0028=( u0029=) u002a=* u002b=+ u002c=, u002d=- u002e=.""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,44,"""u002f=/ u0030=0 u0031=1 u0032=2 u0033=3 u0034=4 u0035=5 u0036=6""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,56,"""u0037=7 u0038=8 u0039=9 u003a=: u003b=; u003c=< u003d== u003e=>""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,68,"""u003f=? u0040=@ u0041=A u0042=B u0043=C u0044=D u0045=E u0046=F""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,80,"""u0047=G u0048=H u0049=I u004a=J u004b=K u004c=L u004d=M u004e=N""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,92,"""u004f=O u0050=P u0051=Q u0052=R u0053=S u0054=T u0055=U u0056=V""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,104,"""u0057=W u0058=X u0059=Y u005a=Z u005b=[ u005c=\\ u005d=] u005e=^""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,116,"""u005f=_ u0060=` u0061=a u0062=b u0063=c u0064=d u0065=e u0066=f""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,128,"""u0067=g u0068=h u0069=i u006a=j u006b=k u006c=l u006d=m u006e=n""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,140,"""u006f=o u0070=p u0071=q u0072=r u0073=s u0074=t u0075=u u0076=v""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,152,"""u0077=w u0078=x u0079=y u007a=z u007b={ u007c=| u007d=} u007e=~""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,164,"""u00a0=\xc2\xa0 u00a1=\xc2\xa1 u00a2=\xc2\xa2 u00a3=\xc2\xa3 u00a4=\xc2\xa4 u00a5=\xc2\xa5 u00a6=\xc2\xa6 u00a7=\xc2\xa7""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,176,"""u00a8=\xc2\xa8 u00a9=\xc2\xa9 u00aa=\xc2\xaa u00ab=\xc2\xab u00ac=\xc2\xac u00ad=\xc2\xad u00ae=\xc2\xae u00af=\xc2\xaf""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,188,"""u00b0=\xc2\xb0 u00b1=\xc2\xb1 u00b2=\xc2\xb2 u00b3=\xc2\xb3 u00b4=\xc2\xb4 u00b5=\xc2\xb5 u00b6=\xc2\xb6 u00b7=\xc2\xb7""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,200,"""u00b8=\xc2\xb8 u00b9=\xc2\xb9 u00ba=\xc2\xba u00bb=\xc2\xbb u00bc=\xc2\xbc u00bd=\xc2\xbd u00be=\xc2\xbe u00bf=\xc2\xbf""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,212,"""u00c0=\xc3\x80 u00c1=\xc3\x81 u00c2=\xc3\x82 u00c3=\xc3\x83 u00c4=\xc3\x84 u00c5=\xc3\x85 u00c6=\xc3\x86 u00c7=\xc3\x87""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,224,"""u00c8=\xc3\x88 u00c9=\xc3\x89 u00ca=\xc3\x8a u00cb=\xc3\x8b u00cc=\xc3\x8c u00cd=\xc3\x8d u00ce=\xc3\x8e u00cf=\xc3\x8f""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,236,"""u00d0=\xc3\x90 u00d1=\xc3\x91 u00d2=\xc3\x92 u00d3=\xc3\x93 u00d4=\xc3\x94 u00d5=\xc3\x95 u00d6=\xc3\x96 u00d7=\xc3\x97""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,248,"""u00d8=\xc3\x98 u00d9=\xc3\x99 u00da=\xc3\x9a u00db=\xc3\x9b u00dc=\xc3\x9c u00dd=\xc3\x9d u00de=\xc3\x9e u00df=\xc3\x9f""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,260,"""u00e0=\xc3\xa0 u00e1=\xc3\xa1 u00e2=\xc3\xa2 u00e3=\xc3\xa3 u00e4=\xc3\xa4 u00e5=\xc3\xa5 u00e6=\xc3\xa6 u00e7=\xc3\xa7""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,272,"""u00e8=\xc3\xa8 u00e9=\xc3\xa9 u00ea=\xc3\xaa u00eb=\xc3\xab u00ec=\xc3\xac u00ed=\xc3\xad u00ee=\xc3\xae u00ef=\xc3\xaf""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,284,"""u00f0=\xc3\xb0 u00f1=\xc3\xb1 u00f2=\xc3\xb2 u00f3=\xc3\xb3 u00f4=\xc3\xb4 u00f5=\xc3\xb5 u00f6=\xc3\xb6 u00f7=\xc3\xb7""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,296,"""u00f8=\xc3\xb8 u00f9=\xc3\xb9 u00fa=\xc3\xba u00fb=\xc3\xbb u00fc=\xc3\xbc u00fd=\xc3\xbd u00fe=\xc3\xbe u00ff=\xc3\xbf""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,308,"""u0100=\xc4\x80 u0101=\xc4\x81 u0102=\xc4\x82 u0103=\xc4\x83 u0104=\xc4\x84 u0105=\xc4\x85 u0106=\xc4\x86 u0107=\xc4\x87""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,320,"""u0108=\xc4\x88 u0109=\xc4\x89 u010a=\xc4\x8a u010b=\xc4\x8b u010c=\xc4\x8c u010d=\xc4\x8d u010e=\xc4\x8e u010f=\xc4\x8f""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,332,"""u2019=\xe2\x80\x99 u201a=\xe2\x80\x9a u201b=\xe2\x80\x9b u201c=\xe2\x80\x9c u201d=\xe2\x80\x9d u201e=\xe2\x80\x9e u201f=\xe2\x80\x9f u2020=\xe2\x80\xa0""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,344,"""u2021=\xe2\x80\xa1 u2022=\xe2\x80\xa2 u2023=\xe2\x80\xa3 u2024=\xe2\x80\xa4 u2025=\xe2\x80\xa5 u2026=\xe2\x80\xa6 u2027=\xe2\x80\xa7 u2028=\xe2\x80\xa8""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,356,"""u2029=\xe2\x80\xa9 u202a=\xe2\x80\xaa u202b=\xe2\x80\xab u202c=\xe2\x80\xac u202d=\xe2\x80\xad u202e=\xe2\x80\xae u202f=\xe2\x80\xaf u2030=\xe2\x80\xb0""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,368,"""u2031=\xe2\x80\xb1 u2032=\xe2\x80\xb2 u2033=\xe2\x80\xb3 u2034=\xe2\x80\xb4 u2035=\xe2\x80\xb5 u2036=\xe2\x80\xb6 u2037=\xe2\x80\xb7 u2038=\xe2\x80\xb8""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,380,"""u2039=\xe2\x80\xb9 u203a=\xe2\x80\xba u203b=\xe2\x80\xbb u203c=\xe2\x80\xbc u203d=\xe2\x80\xbd u203e=\xe2\x80\xbe u203f=\xe2\x80\xbf u2040=\xe2\x81\x80""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,392,"""u2041=\xe2\x81\x81 u2042=\xe2\x81\x82 u2043=\xe2\x81\x83 u2044=\xe2\x81\x84 u2045=\xe2\x81\x85 u2046=\xe2\x81\x86 u2047=\xe2\x81\x87 u2048=\xe2\x81\x88""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,452,"""u206f=\xe2\x81\xaf u2070=\xe2\x81\xb0 u2071=\xe2\x81\xb1 u2074=\xe2\x81\xb4 u2075=\xe2\x81\xb5 u2076=\xe2\x81\xb6 u2077=\xe2\x81\xb7 u2078=\xe2\x81\xb8""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,464,"""u2079=\xe2\x81\xb9 u207a=\xe2\x81\xba u207b=\xe2\x81\xbb u207c=\xe2\x81\xbc u207d=\xe2\x81\xbd u207e=\xe2\x81\xbe u207f=\xe2\x81\xbf u2080=\xe2\x82\x80""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,476,"""u2081=\xe2\x82\x81 u2082=\xe2\x82\x82 u2083=\xe2\x82\x83 u2084=\xe2\x82\x84 u2085=\xe2\x82\x85 u2086=\xe2\x82\x86 u2087=\xe2\x82\x87 u2088=\xe2\x82\x88""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,488,"""u2089=\xe2\x82\x89 u208a=\xe2\x82\x8a u208b=\xe2\x82\x8b u208c=\xe2\x82\x8c u208d=\xe2\x82\x8d u208e=\xe2\x82\x8e u2090=\xe2\x82\x90 u2091=\xe2\x82\x91""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,500,"""u2092=\xe2\x82\x92 u2093=\xe2\x82\x93 u2094=\xe2\x82\x94 u20a0=\xe2\x82\xa0 u20a1=\xe2\x82\xa1 u20a2=\xe2\x82\xa2 u20a3=\xe2\x82\xa3 u20a4=\xe2\x82\xa4""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,512,"""u20a5=\xe2\x82\xa5 u20a6=\xe2\x82\xa6 u20a7=\xe2\x82\xa7 u20a8=\xe2\x82\xa8 u20a9=\xe2\x82\xa9 u20aa=\xe2\x82\xaa u20ab=\xe2\x82\xab u20ac=\xe2\x82\xac""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,524,"""u20ad=\xe2\x82\xad u20ae=\xe2\x82\xae u20af=\xe2\x82\xaf u20b0=\xe2\x82\xb0 u20b1=\xe2\x82\xb1 u20b2=\xe2\x82\xb2 u20b3=\xe2\x82\xb3 u20b4=\xe2\x82\xb4""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,536,"""u20b5=\xe2\x82\xb5 u20d0=\xe2\x83\x90 u20d1=\xe2\x83\x91 u20d2=\xe2\x83\x92 u20d3=\xe2\x83\x93 u20d4=\xe2\x83\x94 u20d5=\xe2\x83\x95 u20d6=\xe2\x83\x96""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,548,"""u20d7=\xe2\x83\x97 u20d8=\xe2\x83\x98 u20d9=\xe2\x83\x99 u20da=\xe2\x83\x9a u20db=\xe2\x83\x9b u20dc=\xe2\x83\x9c u20dd=\xe2\x83\x9d u20de=\xe2\x83\x9e""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,560,"""u20df=\xe2\x83\x9f u20e0=\xe2\x83\xa0 u20e1=\xe2\x83\xa1 u20e2=\xe2\x83\xa2 u20e3=\xe2\x83\xa3 u20e4=\xe2\x83\xa4 u20e5=\xe2\x83\xa5 u20e6=\xe2\x83\xa6""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,572,"""u20e7=\xe2\x83\xa7 u20e8=\xe2\x83\xa8 u20e9=\xe2\x83\xa9 u20ea=\xe2\x83\xaa u20eb=\xe2\x83\xab u2100=\xe2\x84\x80 u2101=\xe2\x84\x81 u2102=\xe2\x84\x82""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,584,"""u2103=\xe2\x84\x83 u2104=\xe2\x84\x84 u2105=\xe2\x84\x85 u2106=\xe2\x84\x86 u2107=\xe2\x84\x87 u2108=\xe2\x84\x88 u2109=\xe2\x84\x89 u210a=\xe2\x84\x8a""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,596,"""u210b=\xe2\x84\x8b u210c=\xe2\x84\x8c u210d=\xe2\x84\x8d u210e=\xe2\x84\x8e u210f=\xe2\x84\x8f u2110=\xe2\x84\x90 u2111=\xe2\x84\x91 u2112=\xe2\x84\x92""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,608,"""u2113=\xe2\x84\x93 u2114=\xe2\x84\x94 u2115=\xe2\x84\x95 u2116=\xe2\x84\x96 u2117=\xe2\x84\x97 u2118=\xe2\x84\x98 u2119=\xe2\x84\x99 u211a=\xe2\x84\x9a""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,620,"""u211b=\xe2\x84\x9b u211c=\xe2\x84\x9c u211d=\xe2\x84\x9d u211e=\xe2\x84\x9e u211f=\xe2\x84\x9f u2120=\xe2\x84\xa0 u2121=\xe2\x84\xa1 u2122=\xe2\x84\xa2""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,632,"""u2123=\xe2\x84\xa3 u2124=\xe2\x84\xa4 u2125=\xe2\x84\xa5 u2126=\xe2\x84\xa6 u2127=\xe2\x84\xa7 u2128=\xe2\x84\xa8 u2129=\xe2\x84\xa9 u212a=\xe2\x84\xaa""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,644,"""u212b=\xe2\x84\xab u212c=\xe2\x84\xac u212d=\xe2\x84\xad u212e=\xe2\x84\xae u212f=\xe2\x84\xaf u2130=\xe2\x84\xb0 u2131=\xe2\x84\xb1 u2132=\xe2\x84\xb2""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,656,"""u2133=\xe2\x84\xb3 u2134=\xe2\x84\xb4 u2135=\xe2\x84\xb5 u2136=\xe2\x84\xb6 u2137=\xe2\x84\xb7 u2138=\xe2\x84\xb8 u2139=\xe2\x84\xb9 u213a=\xe2\x84\xba""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,668,"""u213b=\xe2\x84\xbb u213c=\xe2\x84\xbc u213d=\xe2\x84\xbd u213e=\xe2\x84\xbe u213f=\xe2\x84\xbf u2140=\xe2\x85\x80 u2141=\xe2\x85\x81 u2142=\xe2\x85\x82""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,680,"""u2143=\xe2\x85\x83 u2144=\xe2\x85\x84 u2145=\xe2\x85\x85 u2146=\xe2\x85\x86 u2147=\xe2\x85\x87 u2148=\xe2\x85\x88 u2149=\xe2\x85\x89 u214a=\xe2\x85\x8a""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,692,"""u214b=\xe2\x85\x8b u214c=\xe2\x85\x8c u2153=\xe2\x85\x93 u2154=\xe2\x85\x94 u2155=\xe2\x85\x95 u2156=\xe2\x85\x96 u2157=\xe2\x85\x97 u2158=\xe2\x85\x98""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,704,"""u2159=\xe2\x85\x99 u215a=\xe2\x85\x9a u215b=\xe2\x85\x9b u215c=\xe2\x85\x9c u215d=\xe2\x85\x9d u215e=\xe2\x85\x9e u215f=\xe2\x85\x9f u2160=\xe2\x85\xa0""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,716,"""u2161=\xe2\x85\xa1 u2162=\xe2\x85\xa2 u2163=\xe2\x85\xa3 u2164=\xe2\x85\xa4 u2165=\xe2\x85\xa5 u2166=\xe2\x85\xa6 u2167=\xe2\x85\xa7 u2168=\xe2\x85\xa8""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,728,"""u2169=\xe2\x85\xa9 u216a=\xe2\x85\xaa u216b=\xe2\x85\xab u216c=\xe2\x85\xac u216d=\xe2\x85\xad u216e=\xe2\x85\xae u216f=\xe2\x85\xaf u2170=\xe2\x85\xb0""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)
		self._add(self,String(10,740,"""u2171=\xe2\x85\xb1 u2172=\xe2\x85\xb2 u2173=\xe2\x85\xb3 u2174=\xe2\x85\xb4 u2175=\xe2\x85\xb5 u2176=\xe2\x85\xb6 u2177=\xe2\x85\xb7 u2178=\xe2\x85\xb8""",fontName='arialuni',fontSize=10),name=None,validate=None,desc=None)

if __name__=="__main__": #NORUNTESTS
	TestArialUni().save(formats=['pdf','eps','gif'],outDir='.',fnRoot=None,preview=0)
